==> redux-schema-4.4.2.tgz: package/es/parse/any-object.js <==
var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import finalizeType from '../parse/finalize-type';
import { pathToStr, isArray, isPlainObject } from '../utils';
import { arrayMethods, arrayVirtuals } from './array';
import { hydratePrototype, hydrateInstance } from './hydrate';

export default function anyObject(options, arrayType) {

  arrayType = Boolean(arrayType);

  var self = { options: options },
      kind = arrayType ? 'array' : 'object',
      storedKeys = [],
      storedState = void 0,
      prototype = void 0,
      thisType = void 0;

  if (options.self) {
    self = _extends(options.self, self);
  }

  function isValidObject(value, forceArray) {
    if (!isPlainObject(value) && !isArray(value)) {
      return false;
    }

    if (typeof forceArray !== 'undefined') {
      if (isArray(value) !== forceArray) return false;
    }

    var keys = Object.keys(value),
        propVal = void 0;
    for (var i = 0; i < keys.length; i++) {
      propVal = value[keys[i]];
      if (typeof propVal === 'object' && propVal !== null && !isValidObject(propVal)) {
        return false;
      }
    }
    return true;
  }

  function clone(obj) {
    if (typeof obj !== 'object' || obj === null) {
      return obj;
    }
    var out = isArray(obj) ? [] : {},
        isSchemaObject = obj && obj._meta && obj._meta.type && (obj._meta.type.kind === 'object' || obj._meta.type.kind === 'array'),
        keys = void 0,
        key = void 0,
        value = void 0;

    function getProp(prop) {
      if (isSchemaObject) {
        return obj.get(prop);
      }
      return obj[prop];
    }

    if (isSchemaObject) {
      keys = obj.keys;
    } else {
      keys = Object.keys(obj);
    }

    for (var i = 0; i < keys.length; i++) {
      key = keys[i];
      value = getProp(key);
      if (typeof value === 'object' && value !== null) {
        value = clone(value);
      }
      out[key] = value;
    }
    return out;
  }

  thisType = {
    isType: true,
    name: pathToStr(options.typeMoniker) || arrayType ? 'array' : 'object',
    kind: kind,
    storageKinds: [kind],
    options: options,
    validateData: function validateData(value, instancePath) {
      instancePath = instancePath || options.typeMoniker;
      if (!isValidObject(value, arrayType)) {
        return 'Type of "' + pathToStr(instancePath) + '" data must be ' + kind;
      }
    },
    validateAssign: function validateAssign(value, instancePath) {
      instancePath = instancePath || options.typeMoniker;
      if (!isValidObject(value, arrayType)) {
        return 'Type of "' + pathToStr(instancePath) + '" data must be ' + kind;
      }
    },
    pack: function pack(value) {
      if (!isValidObject(value, arrayType)) {
        throw new TypeError(pathToStr(options.typeMoniker) + ' only accepts simple ' + kind + 's');
      }
      return clone(value);
    },
    unpack: function unpack(store, storePath, instancePath, currentInstance, owner) {
      return hydrateInstance(_extends({}, options, {
        prototype: prototype,
        store: store,
        storePath: storePath,
        instancePath: instancePath,
        currentInstance: currentInstance,
        meta: { owner: owner }
      }));
    },
    getTypeFromPath: function getTypeFromPath(path) {
      return options.typeMoniker.concat(path);
    },
    defaultValue: function defaultValue() {
      return arrayType ? [] : {};
    },

    properties: {},
    methods: {},
    virtuals: {},
    defaultRestProp: function defaultRestProp() {},
    packProp: function packProp(name, value) {
      if (typeof value === 'object' && value !== null && !isValidObject(value)) {
        throw new TypeError(pathToStr(options.typeMoniker.concat(name)) + ' only accepts simple types');
      }
      return clone(value);
    }
  };

  if ('name' in self) {
    delete thisType.name;
  }
  self = _extends(self, finalizeType(thisType));

  prototype = hydratePrototype({
    type: self,
    typePath: options.typeMoniker,
    getter: function getter(name) {
      var meta = this._meta,
          storeValue = meta.store.get(meta.storePath),
          propValue = storeValue[name],
          array = isArray(propValue),
          ix = Number(name),
          type = void 0;

      if (arrayType) {
        if (isNaN(name) || ix % 1 !== 0 || ix < 0 || String(ix) !== String(name) || ix >= this.length) {
          return undefined;
        }
      }

      if (typeof propValue === 'object' && propValue !== null) {
        type = anyObject(_extends({}, options, {
          parent: self,
          self: null,
          typeMoniker: options.typeMoniker.concat(name)
        }), array);
        return meta.store.unpack(type, meta.storePath.concat(name), meta.instancePath.concat(name), null, this);
      } else {
        return propValue;
      }
    },
    setter: function setter(name, value) {
      var meta = this._meta,
          ix = Number(name),
          newState = void 0;
      if (typeof value === 'object' && value === null && !isValidObject(value)) {
        throw new TypeError(pathToStr(options.typeMoniker.concat(name)) + ' only accepts simple types');
      }
      if (arrayType) {
        if (isNaN(name) || ix % 1 !== 0 || ix < 0 || String(ix) !== String(name)) {
          throw new TypeError('Cannot set "' + pathToStr(options.typeMoniker.concat(name)) + '" property on array');
        }
        newState = meta.store.get(meta.storePath);
        if (ix > this.length) {
          newState = newState.slice();
          while (ix > newState.length) {
            newState.push(undefined);
          }
          newState.push(clone(value));
          return meta.store.put(meta.storePath, newState);
        }
      }
      meta.store.put(meta.storePath.concat(name), clone(value));
    },
    keys: function keys() {
      this._meta.store.recordRead(this._meta.storePath);
      var state = this._meta.state;
      if (storedState !== state) {
        storedKeys = Object.keys(state);
        storedState = state;
      }
      return storedKeys;
    },

    methods: arrayType ? arrayMethods : {},
    virtuals: arrayType ? arrayVirtuals : {}
  });

  self.prototype = prototype;

  return self;
}

==> redux-schema-4.8.0.tgz: package/es/parse/any-object.js <==
var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import finalizeType from '../parse/finalize-type';
import { pathToStr, isArray, isPlainObject } from '../utils';
import { arrayMethods, arrayVirtuals } from './array';
import { hydratePrototype, hydrateInstance } from './hydrate';

export default function anyObject(options, arrayType) {

  arrayType = Boolean(arrayType);

  var self = { options: options },
      kind = arrayType ? 'array' : 'object',
      storedKeys = [],
      storedState = void 0,
      prototype = void 0,
      thisType = void 0;

  if (options.self) {
    self = _extends(options.self, self);
  }

  function isValidObject(value, forceArray) {
    if (!isPlainObject(value) && !isArray(value)) {
      return false;
    }

    if (typeof forceArray !== 'undefined') {
      if (isArray(value) !== forceArray) return false;
    }

    var keys = Object.keys(value);
    for (var i = 0; i < keys.length; i++) {
      var propVal = value[keys[i]];
      if (typeof propVal === 'object' && propVal !== null && !isValidObject(propVal)) {
        return false;
      }
    }
    return true;
  }

  function coerceObject(value, forceArray) {
    if (isValidObject(value, forceArray)) return value;
    if (!isPlainObject(value) && !isArray(value)) {
      return forceArray ? [] : {};
    }
    if (typeof forceArray !== 'undefined') {
      if (isArray(value) !== forceArray) return forceArray ? [] : {};
    }

    var keys = Object.keys(value);
    var result = forceArray ? [] : {};
    for (var i = 0; i < keys.length; i++) {
      var key = keys[i];
      var propVal = value[key];
      if (typeof propVal === 'object' && propVal !== null) {
        result[key] = coerceObject(value);
      } else {
        result[key] = value;
      }
    }
    return result;
  }

  function clone(obj) {
    if (typeof obj !== 'object' || obj === null) {
      return obj;
    }
    var out = isArray(obj) ? [] : {},
        isSchemaObject = obj && obj._meta && obj._meta.type && (obj._meta.type.kind === 'object' || obj._meta.type.kind === 'array'),
        keys = void 0,
        key = void 0,
        value = void 0;

    function getProp(prop) {
      if (isSchemaObject) {
        return obj.get(prop);
      }
      return obj[prop];
    }

    if (isSchemaObject) {
      keys = obj.keys;
    } else {
      keys = Object.keys(obj);
    }

    for (var i = 0; i < keys.length; i++) {
      key = keys[i];
      value = getProp(key);
      if (typeof value === 'object' && value !== null) {
        value = clone(value);
      }
      out[key] = value;
    }
    return out;
  }

  thisType = {
    isType: true,
    name: pathToStr(options.typeMoniker) || arrayType ? 'array' : 'object',
    kind: kind,
    storageKinds: [kind],
    options: options,
    validateData: function validateData(value, instancePath) {
      instancePath = instancePath || options.typeMoniker;
      if (!isValidObject(value, arrayType)) {
        return 'Type of "' + pathToStr(instancePath) + '" data must be ' + kind;
      }
    },
    coerceData: function coerceData(value) {
      return coerceObject(value, arrayType);
    },
    validateAssign: function validateAssign(value, instancePath) {
      instancePath = instancePath || options.typeMoniker;
      if (!isValidObject(value, arrayType)) {
        return 'Type of "' + pathToStr(instancePath) + '" data must be ' + kind;
      }
    },
    pack: function pack(value) {
      if (!isValidObject(value, arrayType)) {
        throw new TypeError(pathToStr(options.typeMoniker) + ' only accepts simple ' + kind + 's');
      }
      return clone(value);
    },
    unpack: function unpack(store, storePath, instancePath, currentInstance, owner) {
      return hydrateInstance(_extends({}, options, {
        prototype: prototype,
        store: store,
        storePath: storePath,
        instancePath: instancePath,
        currentInstance: currentInstance,
        meta: { owner: owner }
      }));
    },
    getTypeFromPath: function getTypeFromPath(path) {
      return options.typeMoniker.concat(path);
    },
    defaultValue: function defaultValue() {
      return arrayType ? [] : {};
    },

    properties: {},
    methods: {},
    virtuals: {},
    defaultRestProp: function defaultRestProp() {},
    packProp: function packProp(name, value) {
      if (typeof value === 'object' && value !== null && !isValidObject(value)) {
        throw new TypeError(pathToStr(options.typeMoniker.concat(name)) + ' only accepts simple types');
      }
      return clone(value);
    }
  };

  if ('name' in self) {
    delete thisType.name;
  }
  self = _extends(self, finalizeType(thisType));

  prototype = hydratePrototype({
    type: self,
    typePath: options.typeMoniker,
    getter: function getter(name) {
      var meta = this._meta,
          storeValue = meta.store.get(meta.storePath),
          propValue = storeValue[name],
          array = isArray(propValue),
          ix = Number(name),
          type = void 0;

      if (arrayType) {
        if (isNaN(name) || ix % 1 !== 0 || ix < 0 || String(ix) !== String(name) || ix >= this.length) {
          return undefined;
        }
      }

      if (typeof propValue === 'object' && propValue !== null) {
        type = anyObject(_extends({}, options, {
          parent: self,
          self: null,
          typeMoniker: options.typeMoniker.concat(name)
        }), array);
        return meta.store.unpack(type, meta.storePath.concat(name), meta.instancePath.concat(name), null, this);
      } else {
        return propValue;
      }
    },
    setter: function setter(name, value) {
      var meta = this._meta,
          ix = Number(name),
          newState = void 0;
      if (typeof value === 'object' && value === null && !isValidObject(value)) {
        throw new TypeError(pathToStr(options.typeMoniker.concat(name)) + ' only accepts simple types');
      }
      if (arrayType) {
        if (isNaN(name) || ix % 1 !== 0 || ix < 0 || String(ix) !== String(name)) {
          throw new TypeError('Cannot set "' + pathToStr(options.typeMoniker.concat(name)) + '" property on array');
        }
        newState = meta.store.get(meta.storePath);
        if (ix > this.length) {
          newState = newState.slice();
          while (ix > newState.length) {
            newState.push(undefined);
          }
          newState.push(clone(value));
          return meta.store.put(meta.storePath, newState);
        }
      }
      meta.store.put(meta.storePath.concat(name), clone(value));
    },
    keys: function keys() {
      this._meta.store.recordRead(this._meta.storePath);
      var state = this._meta.state;
      if (storedState !== state) {
        storedKeys = Object.keys(state);
        storedState = state;
      }
      return storedKeys;
    },

    methods: arrayType ? arrayMethods : {},
    virtuals: arrayType ? arrayVirtuals : {}
  });

  self.prototype = prototype;

  return self;
}
